@@ -1,5 +1,5 @@
 """
-Code to access the (read-only, but fast) Node based public iNaturalist API
+Code to access the Node-based iNaturalist API
 See: http://api.inaturalist.org/v1/docs/
 
 Most recent API version tested: 1.3.0
@@ -13,23 +13,17 @@ Functions
 
 """
 from logging import getLogger
-from time import sleep
 from typing import List
+from warnings import warn
 
 import requests
 
 from pyinaturalist import api_docs as docs
 from pyinaturalist.api_requests import get
-from pyinaturalist.constants import (
-    INAT_NODE_API_BASE_URL,
-    PER_PAGE_RESULTS,
-    THROTTLING_DELAY,
-    HistogramResponse,
-    JsonResponse,
-    MultiInt,
-)
+from pyinaturalist.constants import API_V1_BASE_URL, HistogramResponse, JsonResponse, MultiInt
 from pyinaturalist.exceptions import ObservationNotFound, TaxonNotFound
 from pyinaturalist.forge_utils import document_request_params
+from pyinaturalist.pagination import add_paginate_all, paginate_all
 from pyinaturalist.request_params import (
     DEFAULT_OBSERVATION_ATTRS,
     NODE_OBS_ORDER_BY_PROPERTIES,
@@ -42,12 +36,34 @@ from pyinaturalist.response_format import (
     convert_all_coordinates,
     convert_all_place_coordinates,
     convert_all_timestamps,
+    convert_generic_timestamps,
     convert_observation_timestamps,
-    flatten_nested_params,
     format_histogram,
-    format_taxon,
 )
 
+__all__ = [
+    'get_controlled_terms',
+    'get_identifications_by_id',
+    'get_identifications',
+    'get_observation',
+    'get_observation_histogram',
+    'get_observations',
+    'get_observation_species_counts',
+    'get_geojson_observations',
+    'get_observation_observers',
+    'get_observation_identifiers',
+    'get_places_by_id',
+    'get_places_nearby',
+    'get_places_autocomplete',
+    'get_projects',
+    'get_projects_by_id',
+    'get_taxa',
+    'get_taxa_by_id',
+    'get_taxa_autocomplete',
+    'get_user_by_id',
+    'get_users_autocomplete',
+    'search',
+]
 logger = getLogger(__name__)
 
 
@@ -58,7 +74,7 @@ def node_api_get(endpoint: str, **kwargs) -> requests.Response:
         endpoint: The name of an endpoint resource, not including the base URL e.g. 'observations'
         kwargs: Arguments for :py:func:`.api_requests.request`
     """
-    return get(f'{INAT_NODE_API_BASE_URL}{endpoint}', **kwargs)
+    return get(f'{API_V1_BASE_URL}/{endpoint}', **kwargs)
 
 
 # Controlled Terms
@@ -78,32 +94,12 @@ def get_controlled_terms(taxon_id: int = None, user_agent: str = None) -> JsonRe
     Example:
 
         >>> response = get_controlled_terms()
-        >>> # Show a condensed list of terms and values
-        >>> for term in response['results']:
-        >>>     values = [f'    {value["id"]}: {value["label"]}' for value in term['values']]
-        >>>     print(f'{term["id"]}: {term["label"]}' + '\\n'.join(values))
+        >>> print(format_controlled_terms(response))
         1: Life Stage
             2: Adult
             3: Teneral
             4: Pupa
-            5: Nymph
-            6: Larva
-            7: Egg
-            8: Juvenile
-            16: Subimago
-        9: Sex
-            10: Female
-            11: Male
-            20: Cannot Be Determined
-        12: Plant Phenology
-            13: Flowering
-            14: Fruiting
-            15: Flower Budding
-            21: No Evidence of Flowering
-        17: Alive or Dead
-            18: Alive
-            19: Dead
-            20: Cannot Be Determined
+        ...
 
         .. admonition:: Example Response (all terms)
             :class: toggle
@@ -137,6 +133,73 @@ def get_controlled_terms(taxon_id: int = None, user_agent: str = None) -> JsonRe
     return response.json()
 
 
+# Identifications
+# --------------------
+
+
+def get_identifications_by_id(identification_id: MultiInt, user_agent: str = None) -> JsonResponse:
+    """Get one or more identification records by ID.
+
+    **API reference:** https://api.inaturalist.org/v1/docs/#!/Identifications/get_identifications_id
+
+    Example:
+
+        >>> get_identifications_by_id(155554373)
+
+        .. admonition:: Example Response
+            :class: toggle
+
+            .. literalinclude:: ../sample_data/get_identifications.py
+
+    Args:
+        identification_id: Get taxa with this ID. Multiple values are allowed.
+
+    Returns:
+        Response dict containing identification records
+    """
+    r = node_api_get('identifications', ids=identification_id, user_agent=user_agent)
+    r.raise_for_status()
+
+    identifications = r.json()
+    identifications['results'] = convert_all_timestamps(identifications['results'])
+    return identifications
+
+
+@document_request_params([docs._identification_params, docs._pagination, docs._only_id])
+@add_paginate_all(method='page')
+def get_identifications(**params) -> JsonResponse:
+    """Search identifications.
+
+    **API reference:** https://api.inaturalist.org/v1/docs/#!/Identifications/get_identifications
+
+    Example:
+
+        Get all of your own species-level identifications:
+
+        >>> response = get_identifications(user_login='my_username', rank='species')
+        >>> print([f"{i['user']['login']}: {i['taxon_id']} ({i['category']})" for i in response['results']])
+        [155043569] Species: 76465 (leading) added on 2021-02-15 10:46:27-06:00 by jkcook
+        [153668189] Species: 76465 (supporting) added on 2021-02-06 17:43:37+00:00 by jkcook
+        [147500725] Species: 1163860 (improving) added on 2020-12-24 23:52:30+00:00 by jkcook
+        ...
+
+        .. admonition:: Example Response
+            :class: toggle
+
+            .. literalinclude:: ../sample_data/get_identifications.py
+
+    Returns:
+        Response dict containing identification records
+    """
+    params = translate_rank_range(params)
+    r = node_api_get('identifications', params=params)
+    r.raise_for_status()
+
+    identifications = r.json()
+    identifications['results'] = convert_all_timestamps(identifications['results'])
+    return identifications
+
+
 # Observations
 # --------------------
 
@@ -148,7 +211,9 @@ def get_observation(observation_id: int, user_agent: str = None) -> JsonResponse
 
     Example:
 
-        >>> get_observation(16227955)
+        >>> response = get_observation(16227955)
+        >>> print(format_observations(response))
+        [16227955] [493595] Species: Lixus bardanae observed on 2018-09-05 14:06:00+01:00 by niconoe at 54 rue des Badauds
 
         .. admonition:: Example Response
             :class: toggle
@@ -174,7 +239,7 @@ def get_observation(observation_id: int, user_agent: str = None) -> JsonResponse
 
 
 @document_request_params([*docs._get_observations, docs._observation_histogram])
-def get_observation_histogram(user_agent: str = None, **params) -> HistogramResponse:
+def get_observation_histogram(**params) -> HistogramResponse:
     """Search observations and return histogram data for the given time interval
 
     **API reference:** https://api.inaturalist.org/v1/docs/#!/Observations/get_observations_histogram
@@ -223,13 +288,14 @@ def get_observation_histogram(user_agent: str = None, **params) -> HistogramResp
         Dict of ``{time_key: observation_count}``. Keys are ints for 'month of year' and\
         'week of year' intervals, and :py:class:`~datetime.datetime` objects for all other intervals.
     """
-    r = node_api_get('observations/histogram', params=params, user_agent=user_agent)
+    r = node_api_get('observations/histogram', params=params)
     r.raise_for_status()
     return format_histogram(r.json())
 
 
 @document_request_params([*docs._get_observations, docs._pagination, docs._only_id])
-def get_observations(user_agent: str = None, **params) -> JsonResponse:
+@add_paginate_all(method='id')
+def get_observations(**params) -> JsonResponse:
     """Search observations.
 
     **API reference:** http://api.inaturalist.org/v1/docs/#!/Observations/get_observations
@@ -250,9 +316,8 @@ def get_observations(user_agent: str = None, **params) -> JsonResponse:
 
         Get basic info for observations in response:
 
-        >>> from pyinaturalist.response_format import format_observation
-        >>> for obs in response['results']:
-        >>>     print(format_observation(obs))
+        >>> from pyinaturalist.formatters import format_observations
+        >>> print(format_observations(response))
         '[57754375] Species: Danaus plexippus (Monarch) observed by samroom on 2020-08-27 at Railway Ave, Wilcox, SK'
         '[57707611] Species: Danaus plexippus (Monarch) observed by ingridt3 on 2020-08-26 at Michener Dr, Regina, SK'
 
@@ -262,10 +327,10 @@ def get_observations(user_agent: str = None, **params) -> JsonResponse:
             .. literalinclude:: ../sample_data/get_observations_node.py
 
     Returns:
-        JSON response containing observation records
+        Response dict containing observation records
     """
     validate_multiple_choice_param(params, 'order_by', NODE_OBS_ORDER_BY_PROPERTIES)
-    r = node_api_get('observations', params=params, user_agent=user_agent)
+    r = node_api_get('observations', params=params)
     r.raise_for_status()
 
     observations = r.json()
@@ -275,55 +340,16 @@ def get_observations(user_agent: str = None, **params) -> JsonResponse:
     return observations
 
 
-# TODO: Consolidate logic from get_all_*() functions into a generic pagination function
-#       This would have two variations: by page number (most common) and by ID (as seen below)
-@document_request_params([*docs._get_observations, docs._only_id])
-def get_all_observations(user_agent: str = None, **params) -> List[JsonResponse]:
-    """Like :py:func:`get_observations()`, but handles pagination and returns all results in one
-    call. Explicit pagination parameters will be ignored.
-
-    Notes on pagination from the iNaturalist documentation:
-    'The large size of the observations index prevents us from supporting the page parameter when
-    retrieving records from large result sets. If you need to retrieve large numbers of records,
-    use the ``per_page`` and ``id_above`` or ``id_below`` parameters instead.'
-
-    Example:
-
-        >>> observations = get_all_observations(
-        >>>     taxon_name='Danaus plexippus',
-        >>>     created_on='2020-08-27',
-        >>> )
-
-    Returns:
-        Combined list of observation records. Response format is the same as the inner 'results'\
-        object returned by :py:func:`.get_observations()`.
-    """
-    results: List[JsonResponse] = []
-    id_above = 0
-    pagination_params = {
-        **params,
-        **{
-            'order_by': 'id',
-            'order': 'asc',
-            'per_page': PER_PAGE_RESULTS,
-            'user_agent': user_agent,
-        },
-    }
-
-    while True:
-        pagination_params['id_above'] = id_above
-        page_obs = get_observations(**pagination_params)
-        results = results + page_obs.get('results', [])
-
-        if page_obs['total_results'] <= PER_PAGE_RESULTS:
-            return results
-
-        sleep(THROTTLING_DELAY)
-        id_above = results[-1]['id']
+def get_all_observations(**params) -> List[JsonResponse]:
+    """Deprecated; use ``get_observations(page='all')`` instead"""
+    msg = "get_all_observations() is deprecated; please use get_observations(page='all') instead"
+    warn(DeprecationWarning(msg))
+    return paginate_all(get_observations, method='id', **params)['results']
 
 
 @document_request_params([*docs._get_observations, docs._pagination])
-def get_observation_species_counts(user_agent: str = None, **params) -> JsonResponse:
+@add_paginate_all(method='page')
+def get_observation_species_counts(**params) -> JsonResponse:
     """Get all species (or other 'leaf taxa') associated with observations matching the search
     criteria, and the count of observations they are associated with.
     **Leaf taxa** are the leaves of the taxonomic tree, e.g., species, subspecies, variety, etc.
@@ -331,7 +357,12 @@ def get_observation_species_counts(user_agent: str = None, **params) -> JsonResp
     **API reference:** https://api.inaturalist.org/v1/docs/#!/Observations/get_observations_species_counts
 
     Example:
-        >>> get_observation_species_counts(user_login='my_username', quality_grade='research')
+        >>> response = get_observation_species_counts(user_login='my_username', quality_grade='research')
+        >>> print(format_species_counts(response))
+        [62060] Species: Palomena prasina (Green Shield Bug): 10
+        [84804] Species: Graphosoma italicum (European Striped Shield Bug): 8
+        [55727] Species: Cymbalaria muralis (Ivy-leaved toadflax): 3
+        ...
 
         .. admonition:: Example Response
             :class: toggle
@@ -339,66 +370,16 @@ def get_observation_species_counts(user_agent: str = None, **params) -> JsonResp
             .. literalinclude:: ../sample_data/get_observation_species_counts.py
 
     Returns:
-        JSON response containing taxon records with counts
+        Response dict containing taxon records with counts
     """
     r = node_api_get(
         'observations/species_counts',
         params=params,
-        user_agent=user_agent,
     )
     r.raise_for_status()
     return r.json()
 
 
-@document_request_params(docs._get_observations)
-def get_all_observation_species_counts(user_agent: str = None, **params) -> List[JsonResponse]:
-    """Like :py:func:`get_observation_species_counts()`, but handles pagination and returns all
-    results in one call. Explicit pagination parameters will be ignored.
-
-    Notes:
-    While the ``page`` parameter is undocumented for observations/species_counts, it appears to be supported.
-    ``id_above`` and ``id_below`` are not helpful in the context.
-
-    Example:
-        >>> get_all_observation_species_counts(
-        ...     user_agent=None,
-        ...     quality_grade='research',
-        ...     place_id=154695,
-        ...     iconic_taxa='Reptilia',
-        ... )
-
-        .. admonition:: Example Response
-            :class: toggle
-
-            .. literalinclude:: ../sample_data/get_all_observation_species_counts_ex_results.json
-                :language: JSON
-
-    Returns:
-        Combined list of taxon records with counts
-    """
-    results: List[JsonResponse] = []
-    page = 1
-
-    pagination_params = {
-        **params,
-        **{
-            'per_page': PER_PAGE_RESULTS,
-            'user_agent': user_agent,
-        },
-    }
-
-    while True:
-        pagination_params['page'] = page
-        page_obs = get_observation_species_counts(**pagination_params)
-        results = results + page_obs.get('results', [])
-
-        if len(results) == page_obs['total_results']:
-            return results
-
-        sleep(THROTTLING_DELAY)
-        page += 1
-
-
 @document_request_params([*docs._get_observations, docs._geojson_properties])
 def get_geojson_observations(properties: List[str] = None, **params) -> JsonResponse:
     """Get all observation results combined into a GeoJSON ``FeatureCollection``.
@@ -418,29 +399,33 @@ def get_geojson_observations(properties: List[str] = None, **params) -> JsonResp
         A ``FeatureCollection`` containing observation results as ``Feature`` dicts.
     """
     params['mappable'] = True
-    observations = get_all_observations(**params)
+    params['page'] = 'all'
+    response = get_observations(**params)
     return as_geojson_feature_collection(
-        (flatten_nested_params(obs) for obs in observations),
+        response['results'],
         properties=properties if properties is not None else DEFAULT_OBSERVATION_ATTRS,
     )
 
 
 @document_request_params([*docs._get_observations, docs._pagination])
-def get_observation_observers(user_agent: str = None, **params) -> JsonResponse:
+def get_observation_observers(**params) -> JsonResponse:
     """Get observers of observations matching the search criteria and the count of
     observations and distinct taxa of rank species they have observed.
 
     Notes:
-    Options for ``order_by`` are 'observation_count' (default) or 'species_count'.
-
-    ``GET /observations/observers`` API node is buggy. It's currently limited to
-    500 results and using ``order_by=species_count`` may produce unusual results.
-    See this issue for more details: https://github.com/inaturalist/iNaturalistAPI/issues/235
+        * Options for ``order_by`` are 'observation_count' (default) or 'species_count'
+        * This endpoint will only return up to 500 results
+        * See this issue for more details: https://github.com/inaturalist/iNaturalistAPI/issues/235
 
     **API reference:** https://api.inaturalist.org/v1/docs/#!/Observations/get_observations_observers
 
     Example:
-        >>> get_observation_observers(place_id=72645, order_by='species_count')
+        >>> response = get_observation_observers(place_id=72645, order_by='species_count')
+        >>> print(format_users(response, align=True))
+        [1566366 ] fossa1211
+        [674557  ] schurchin
+        [5813    ] fluffberger (Fluff Berger)
+
 
         .. admonition:: Example Response
             :class: toggle
@@ -449,29 +434,33 @@ def get_observation_observers(user_agent: str = None, **params) -> JsonResponse:
                 :language: JSON
 
     Returns:
-        JSON response of observers
+        Response dict of observers
     """
-
-    params.setdefault('per_page', 500)  # patch for API issue #235
+    params.setdefault('per_page', 500)
 
     r = node_api_get(
         'observations/observers',
         params=params,
-        user_agent=user_agent,
     )
     r.raise_for_status()
     return r.json()
 
 
 @document_request_params([*docs._get_observations, docs._pagination])
-def get_observation_identifiers(user_agent: str = None, **params) -> JsonResponse:
+def get_observation_identifiers(**params) -> JsonResponse:
     """Get identifiers of observations matching the search criteria and the count of
     observations they have identified. By default, results are sorted by ID count in descending.
 
     **API reference:** https://api.inaturalist.org/v1/docs/#!/Observations/get_observations_identifiers
 
+    Note: This endpoint will only return up to 500 results.
+
     Example:
-        >>> get_observation_identifiers(place_id=72645)
+        >>> response = get_observation_identifiers(place_id=72645)
+        >>> print(format_users(response, align=True))
+        [409010  ] jdoe42 (Jane Doe)
+        [691216  ] jbrown252 (James Brown)
+        [3959037 ] tnsparkleberry
 
         .. admonition:: Example Response
             :class: toggle
@@ -480,19 +469,13 @@ def get_observation_identifiers(user_agent: str = None, **params) -> JsonRespons
                 :language: JSON
 
     Returns:
-        JSON response of identifiers
-
-    Notes:
-    The ``GET /observations/identifiers`` API node is currently limited to
-    500 results. See this issue for more details: https://github.com/inaturalist/iNaturalistAPI/issues/236
+        Response dict of identifiers
     """
-
-    params.setdefault('per_page', 500)  # patch until issue #236 is resolved
+    params.setdefault('per_page', 500)
 
     r = node_api_get(
         'observations/identifiers',
         params=params,
-        user_agent=user_agent,
     )
     r.raise_for_status()
     return r.json()
@@ -510,11 +493,9 @@ def get_places_by_id(place_id: MultiInt, user_agent: str = None) -> JsonResponse
 
     Example:
         >>> response = get_places_by_id([67591, 89191])
-        >>> print({p['id']: p['name'] for p in  response['results']})
-        {
-            67591: 'Riversdale Beach',
-            89191: 'Conservation Area Riversdale',
-        }
+        >>> print(format_places(response))
+        [89191] Conservation Area Riversdale
+        [67591] Riversdale Beach
 
         .. admonition:: Example Response
             :class: toggle
@@ -525,7 +506,7 @@ def get_places_by_id(place_id: MultiInt, user_agent: str = None) -> JsonResponse
         place_id: Get a place with this ID. Multiple values are allowed.
 
     Returns:
-        JSON response containing place records
+        Response dict containing place records
     """
     r = node_api_get('places', ids=place_id, user_agent=user_agent)
     r.raise_for_status()
@@ -537,10 +518,9 @@ def get_places_by_id(place_id: MultiInt, user_agent: str = None) -> JsonResponse
 
 
 @document_request_params([docs._bounding_box, docs._name])
-def get_places_nearby(user_agent: str = None, **params) -> JsonResponse:
+def get_places_nearby(**params) -> JsonResponse:
     """
-    Given an bounding box, and an optional name query, return standard iNaturalist curator approved
-    and community non-curated places nearby
+    Given an bounding box, and an optional name query, return places nearby
 
     **API reference:** https://api.inaturalist.org/v1/docs/#!/Places/get_places_nearby
 
@@ -548,37 +528,26 @@ def get_places_nearby(user_agent: str = None, **params) -> JsonResponse:
         >>> bounding_box = (150.0, -50.0, -149.999, -49.999)
         >>> response = get_places_nearby(*bounding_box)
 
-        Show basic info for standard (curated) places in response:
+        Response is split into standard (curated) places and community (non-curated) places:
 
-        >>> print({p['id']: p['name'] for p in  response['results']['standard']})
-        {
-            97394: 'North America',
-            97395: 'Asia',
-            97393: 'Oceania',
-            97392: 'Africa',
-            97391: 'Europe',
-            97389: 'South America',
-            7161:  'Russia',
-            1:     'United States',
-            6712:  'Canada',
-            10316: 'United States Minor Outlying Islands',
-        }
+        >>> print(len(response['results']['standard']))
+        10
+        >>> print(len(response['results']['community']))
+        10
 
-        Show basic info for community-contributed places in response:
+        Show basic info for all places in response:
 
-        >>> print({p['id']: p['name'] for p in  response['results']['community']})
-        {
-            11770:  'Mehedinti',
-            119755: 'Mahurangi College',
-            150981: 'Ceap Breatainn',
-            136758: 'Willapa Hills (US EPA Level IV Ecoregion)',
-            119694: 'Tetlin National Wildlife Refuge',
-            70630:  'Murray - Sunset',
-            141915: 'Fundy Pollinator Trail',
-            72346:  'Sucunduri',
-            50505:  'Great Salt Lake',
-            72230:  'Rio Novo',
-         }
+        >>> print(format_places(response, align=True))
+        Standard:
+        [97394  ] North America
+        [97395  ] Asia
+        [97393  ] Oceania
+        ...
+        Community:
+        [166719  ] Burgenland (accurate border)
+        [11770   ] Mehedinti
+        [119755  ] Mahurangi College
+        ...
 
         .. admonition:: Example Response
             :class: toggle
@@ -586,27 +555,31 @@ def get_places_nearby(user_agent: str = None, **params) -> JsonResponse:
             .. literalinclude:: ../sample_data/get_places_nearby.py
 
     Returns:
-        JSON response containing place records, divided into 'standard' and 'community' places.
+        Response dict containing place records, divided into 'standard' and 'community' places.
     """
-    r = node_api_get('places/nearby', params=params, user_agent=user_agent)
+    r = node_api_get('places/nearby', params=params)
     r.raise_for_status()
     return convert_all_place_coordinates(r.json())
 
 
-def get_places_autocomplete(q: str, user_agent: str = None) -> JsonResponse:
+@document_request_params([docs._search_query, docs._pagination])
+@add_paginate_all(method='autocomplete')
+def get_places_autocomplete(q: str = None, **params) -> JsonResponse:
     """Given a query string, get places with names starting with the search term
 
     **API reference:** https://api.inaturalist.org/v1/docs/#!/Places/get_places_autocomplete
 
+    **Note:** This endpoint accepts a ``per_page`` param, up to a max of 20 (default 10). Pages
+    beyond the first page cannot be retrieved. Use ``page=all`` to attempt to retrieve additional
+    results. See :py:func:`.paginate_autocomplete` for more info.
+
     Example:
         >>> response = get_places_autocomplete('Irkutsk')
-        >>> print({p['id']: p['name'] for p in  response['results']})
-        {
-            11803:  'Irkutsk',
-            41853:  'Irkutsk',
-            41854:  'Irkutskiy rayon',
-            163077: 'Irkutsk agglomeration',
-        }
+        >>> print(format_places(response))
+        [11803   ] Irkutsk
+        [41854   ] Irkutskiy rayon
+        [166186  ] Irkutsk Oblast - ADD
+        [163077  ] Irkutsk agglomeration
 
         .. admonition:: Example Response
             :class: toggle
@@ -617,9 +590,9 @@ def get_places_autocomplete(q: str, user_agent: str = None) -> JsonResponse:
         q: Name must begin with this value
 
     Returns:
-        JSON response containing place records
+        Response dict containing place records
     """
-    r = node_api_get('places/autocomplete', params={'q': q}, user_agent=user_agent)
+    r = node_api_get('places/autocomplete', params={'q': q, **params})
     r.raise_for_status()
 
     # Convert coordinates to floats
@@ -633,7 +606,8 @@ def get_places_autocomplete(q: str, user_agent: str = None) -> JsonResponse:
 
 
 @document_request_params([docs._projects_params, docs._pagination])
-def get_projects(user_agent: str = None, **params) -> JsonResponse:
+@add_paginate_all(method='page')
+def get_projects(**params) -> JsonResponse:
     """Given zero to many of following parameters, get projects matching the search criteria.
 
     **API reference:** https://api.inaturalist.org/v1/docs/#!/Projects/get_projects
@@ -652,14 +626,12 @@ def get_projects(user_agent: str = None, **params) -> JsonResponse:
 
         Show basic info for projects in response:
 
-        >>> print({p['id']: p['title'] for p in response['results']})
-        {
-            8291:  'PNW Invasive Plant EDDR',
-            19200: 'King County (WA) Noxious and Invasive Weeds',
-            8527:  'WA Invasives',
-            2344:  'Invasive & Huntable Animals',
-            6432:  'CBWN Invasive Plants',
-        }
+        >>> print(format_projects(response, align=True))
+        [8291    ] PNW Invasive Plant EDDR
+        [19200   ] King County (WA) Noxious and Invasive Weeds
+        [102925  ] Keechelus/Kachess Invasive Plants
+        ...
+
 
         .. admonition:: Example Response
             :class: toggle
@@ -667,10 +639,10 @@ def get_projects(user_agent: str = None, **params) -> JsonResponse:
             .. literalinclude:: ../sample_data/get_projects.py
 
     Returns:
-        JSON response containing project records
+        Response dict containing project records
     """
     validate_multiple_choice_param(params, 'order_by', PROJECT_ORDER_BY_PROPERTIES)
-    r = node_api_get('projects', params=params, user_agent=user_agent)
+    r = node_api_get('projects', params=params)
     r.raise_for_status()
 
     response = r.json()
@@ -688,7 +660,10 @@ def get_projects_by_id(
 
     Example:
 
-        >>> get_projects_by_id([8348, 6432])
+        >>> response = get_projects_by_id([8348, 6432])
+        >>> print(format_projects(response))
+        [8348] Tucson High Native and Invasive Species Inventory
+        [6432] CBWN Invasive Plants
 
         .. admonition:: Example Response
             :class: toggle
@@ -701,7 +676,7 @@ def get_projects_by_id(
             object instead of simply an ID
 
     Returns:
-        JSON response containing project records
+        Response dict containing project records
     """
     r = node_api_get(
         'projects',
@@ -721,8 +696,9 @@ def get_projects_by_id(
 # --------------------
 
 
-@document_request_params([docs._taxon_params, docs._taxon_id_params])
-def get_taxa(user_agent: str = None, **params) -> JsonResponse:
+@document_request_params([docs._taxon_params, docs._taxon_id_params, docs._pagination])
+@add_paginate_all(method='page')
+def get_taxa(**params) -> JsonResponse:
     """Given zero to many of following parameters, get taxa matching the search criteria.
 
     **API reference:** https://api.inaturalist.org/v1/docs/#!/Taxa/get_taxa
@@ -730,8 +706,11 @@ def get_taxa(user_agent: str = None, **params) -> JsonResponse:
     Example:
 
         >>> response = get_taxa(q='vespi', rank=['genus', 'family'])
-        >>> print({taxon['id']: taxon['name'] for taxon in response['results']})
-        {52747: 'Vespidae', 84737: 'Vespina', 92786: 'Vespicula', 646195: 'Vespiodes', ...}
+        >>> print(format_taxa(response))
+        [52747] Family: Vespidae (Hornets, Paper Wasps, Potter Wasps, and Allies)
+        [92786] Genus: Vespicula
+        [646195] Genus: Vespiodes
+        ...
 
         .. admonition:: Example Response
             :class: toggle
@@ -740,10 +719,10 @@ def get_taxa(user_agent: str = None, **params) -> JsonResponse:
                 :language: JSON
 
     Returns:
-        JSON response containing taxon records
+        Response dict containing taxon records
     """
     params = translate_rank_range(params)
-    r = node_api_get('taxa', params=params, user_agent=user_agent)
+    r = node_api_get('taxa', params=params)
     r.raise_for_status()
 
     taxa = r.json()
@@ -777,7 +756,7 @@ def get_taxa_by_id(taxon_id: MultiInt, user_agent: str = None) -> JsonResponse:
         taxon_id: Get taxa with this ID. Multiple values are allowed.
 
     Returns:
-        JSON response containing taxon records
+        Response dict containing taxon records
     """
     r = node_api_get('taxa', ids=taxon_id, user_agent=user_agent)
     r.raise_for_status()
@@ -787,9 +766,9 @@ def get_taxa_by_id(taxon_id: MultiInt, user_agent: str = None) -> JsonResponse:
     return taxa
 
 
-@document_request_params([docs._taxon_params, docs._minify])
-def get_taxa_autocomplete(user_agent: str = None, **params) -> JsonResponse:
-    """Given a query string, returns taxa with names starting with the search term
+@document_request_params([docs._taxon_params])
+def get_taxa_autocomplete(**params) -> JsonResponse:
+    """Given a query string, return taxa with names starting with the search term
 
     **API reference:** https://api.inaturalist.org/v1/docs/#!/Taxa/get_taxa_autocomplete
 
@@ -798,31 +777,143 @@ def get_taxa_autocomplete(user_agent: str = None, **params) -> JsonResponse:
 
     Example:
 
+        Get just the name of the first matching taxon:
+
         >>> response = get_taxa_autocomplete(q='vespi')
-        >>> first_result = response['results'][0]
-        >>> print(first_result['rank'], first_result['name'])
-        'family Vespidae'
+        >>> print(response['results'][0]['name'])
+        'Vespidae'
+
+        Get basic info for taxa in response:
+
+        >>> print(format_taxa(response, align=True))
+        [52747   ]       Family: Vespidae (Hornets, Paper Wasps, Potter Wasps, and Allies)
+        [84738   ]    Subfamily: Vespinae (Hornets and Yellowjackets)
+        [131878  ]      Species: Nicrophorus vespillo (Vespillo Burying Beetle)
+
+        If you get unexpected matches, the search likely matched a synonym, either in the form of a
+        common name or an alternative classification. Check the ``matched_term`` property for more
+        info. For example:
+
+        >>> first_result = get_taxa_autocomplete(q='zygoca')['results'][0]
+        >>> first_result["name"]
+        "Schlumbergera truncata"  # This doesn't look like our search term!
+        >>> first_result["matched_term"]
+        "Zygocactus truncatus"    # ...Because it matched an older synonym for Schlumbergera
 
         .. admonition:: Example Response
             :class: toggle
 
-            .. literalinclude:: ../sample_data/get_taxa_autocomplete.json
-                :language: JSON
+            .. literalinclude:: ../sample_data/get_taxa_autocomplete.py
 
-        .. admonition:: Example Response (with **minify=True**)
+        .. admonition:: Example Response (formatted)
             :class: toggle
 
-            .. literalinclude:: ../sample_data/get_taxa_autocomplete_minified.json
-                :language: JSON
+            .. literalinclude:: ../sample_data/get_taxa_autocomplete_minified.py
 
     Returns:
-        JSON response containing taxon records
+        Response dict containing taxon records
     """
     params = translate_rank_range(params)
-    r = node_api_get('taxa/autocomplete', params=params, user_agent=user_agent)
+    r = node_api_get('taxa/autocomplete', params=params)
     r.raise_for_status()
-    json_response = r.json()
+    return r.json()
 
-    if params.get('minify'):
-        json_response['results'] = [format_taxon(t, align=True) for t in json_response['results']]
-    return json_response
+
+# Users
+# --------------------
+
+
+def get_user_by_id(user_id: int, user_agent: str = None) -> JsonResponse:
+    """Get a user by ID.
+
+    **API reference:** https://api.inaturalist.org/v1/docs/#!/Users/get_users_id
+
+    Args:
+        user_id: Get the user with this ID. Only a single ID is allowed per request.
+
+    Example:
+
+        >>> response = get_user_by_id(123456)
+        >>> print(format_users(response))
+        [1234] my_username
+
+        .. admonition:: Example Response
+            :class: toggle
+
+            .. literalinclude:: ../sample_data/get_user_by_id.py
+
+    Returns:
+        Response dict containing user record
+    """
+    r = node_api_get('users', ids=[user_id], user_agent=user_agent)
+    r.raise_for_status()
+    results = r.json()['results']
+    if not results:
+        return {}
+    return convert_generic_timestamps(results[0])
+
+
+@document_request_params([docs._search_query, docs._project_id, docs._pagination])
+def get_users_autocomplete(q: str, **params) -> JsonResponse:
+    """Given a query string, return users with names or logins starting with the search term
+
+    **API reference:** https://api.inaturalist.org/v1/docs/#!/Users/get_users_autocomplete
+
+    Note: Pagination is supported; default page size is 6, and max is 100.
+
+    Example:
+
+        >>> response = get_taxa_autocomplete(q='my_userna')
+        >>> print(format_users(response))
+        [1234] my_username
+        [12345] my_username_2
+
+        .. admonition:: Example Response
+            :class: toggle
+
+            .. literalinclude:: ../sample_data/get_users_autocomplete.py
+
+    Returns:
+        Response dict containing user records
+    """
+    r = node_api_get('users/autocomplete', params={'q': q, **params})
+    r.raise_for_status()
+    users = r.json()
+    users['results'] = convert_all_timestamps(users['results'])
+    return users
+
+
+# Main Search
+# --------------------
+
+
+@document_request_params([docs._search_params, docs._pagination])
+def search(q: str, **params) -> JsonResponse:
+    """A unified search endpoint for places, projects, taxa, and/or users
+
+    **API reference:** https://api.inaturalist.org/v1/docs/#!/Search/get_search
+
+    Example:
+
+        >>> response = search(q='odonat')
+        >>> print(format_search_results(response, align=True))
+        [Taxon  ] [47792   ] Order: Odonata (Dragonflies and Damselflies)
+        [Place  ] [113562  ] Odonates of Peninsular India and Sri Lanka
+        [Project] [9978    ] Ohio Dragonfly Survey  (Ohio Odonata Survey)
+        [User   ] [113886  ] odonatanb (Gilles Belliveau)
+
+
+        .. admonition:: Example Response
+            :class: toggle
+
+            .. literalinclude:: ../sample_data/get_search.py
+
+    Returns:
+        Response dict containing search results
+    """
+    r = node_api_get('search', params={'q': q, **params})
+    r.raise_for_status()
+    search_results = r.json()
+    search_results['results'] = convert_all_timestamps(search_results['results'])
+    search_results['results'] = convert_all_coordinates(search_results['results'])
+    return search_results
