@@ -4,19 +4,45 @@ Pytest will also automatically pick up any fixtures defined here.
 """
 import json
 import logging
+import os
 import re
-from inspect import getmembers, isfunction, signature, Parameter
-from os.path import abspath, dirname, join
+from inspect import Parameter, getmembers, isfunction, signature
+from os.path import join
 from unittest.mock import MagicMock
 
-HTTP_FUNC_PATTERN = re.compile(r"(get|put|post|delete)_.+")
-SAMPLE_DATA_DIR = abspath(join(dirname(__file__), "sample_data"))
+# If ipdb is installed, register it as the default debugger
+from pyinaturalist.constants import SAMPLE_DATA_DIR
+
+try:
+    import ipdb  # noqa: F401
+
+    os.environ['PYTHONBREAKPOINT'] = 'ipdb.set_trace'
+except ImportError:
+    pass
+
+
+HTTP_FUNC_PATTERN = re.compile(r'(get|put|post|delete)_.+')
+
+MOCK_CREDS_ENV = {
+    'INAT_USERNAME': 'valid_username',
+    'INAT_PASSWORD': 'valid_password',
+    'INAT_APP_ID': 'valid_app_id',
+    'INAT_APP_SECRET': 'valid_app_secret',
+}
+MOCK_CREDS_OAUTH = {
+    'username': 'valid_username',
+    'password': 'valid_password',
+    'client_id': 'valid_app_id',
+    'client_secret': 'valid_app_secret',
+}
+
 # Enable logging for urllib and other external loggers
-logging.basicConfig(level="INFO")
+logging.basicConfig(level='INFO')
+logging.getLogger('pyinaturalist').setLevel('DEBUG')
 
 
 def get_module_functions(module):
-    """ Get all functions belonging to a module (excluding imports) """
+    """Get all functions belonging to a module (excluding imports)"""
     return {
         name: member
         for name, member in getmembers(module)
@@ -25,7 +51,7 @@ def get_module_functions(module):
 
 
 def get_module_http_functions(module):
-    """ Get all functions belonging to a module and prefixed with an HTTP method """
+    """Get all functions belonging to a module and prefixed with an HTTP method"""
     return {
         name: func
         for name, func in get_module_functions(module).items()
@@ -52,8 +78,8 @@ def _sample_data_path(filename):
 
 
 def load_sample_data(filename):
-    with open(_sample_data_path(filename), encoding="utf-8") as fh:
-        if filename.endswith("json"):
+    with open(_sample_data_path(filename), encoding='utf-8') as fh:
+        if filename.endswith('json'):
             return json.load(fh)
         else:
             return fh.read()
