@@ -1,4 +1,6 @@
-from pyinaturalist.forge_utils import document_request_params, copy_docstrings, copy_signatures
+from pyinaturalist.forge_utils import copy_docstrings
+
+# from pyinaturalist.forge_utils import copy_signatures, document_request_params
 
 
 def test_document_request_params():
@@ -37,7 +39,7 @@ Returns:
 
 def test_copy_docstrings__without_return_desc():
     def func():
-        """ Test function. """
+        """Test function."""
 
     expected_docstring = """
 Test function.
